]> git.sesse.net Git - cubemap/blobdiff - stream.h
Store multiple suitable starting points for each stream.
[cubemap] / stream.h
index 9a9982a3541d489f72b63a18228f1e1d1910acca..c98feb46e3bb2e09f5913c1ea31a018e02680075 100644 (file)
--- a/stream.h
+++ b/stream.h
@@ -8,6 +8,7 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <deque>
 #include <string>
 #include <vector>
 
@@ -75,9 +76,9 @@ struct Stream {
        // than <backlog_size>, since the buffer wraps.
        size_t bytes_received;
 
-       // The last point in the stream that is suitable to start new clients at
-       // (after having sent the header). -1 if no such point exists yet.
-       ssize_t last_suitable_starting_point;
+       // A list of points in the stream that is suitable to start new clients at
+       // (after having sent the header). Empty if no such point exists yet.
+       std::deque<size_t> suitable_starting_points;
        
        // Clients that are in SENDING_DATA, but that we don't listen on,
        // because we currently don't have any data for them.
@@ -117,8 +118,14 @@ private:
 
        // Adds data directly to the stream file descriptor, without adding headers or
        // going through <queued_data>.
-       // You should hold the owning Server's <mutex>.
+       // You should hold the owning Server's <mutex>, and probably call
+       // remove_obsolete_starting_points() afterwards.
        void add_data_raw(const std::vector<DataElement> &data);
+
+       // Remove points from <suitable_starting_points> that are no longer
+       // in the backlog.
+       // You should hold the owning Server's <mutex>.
+       void remove_obsolete_starting_points();
 };
 
 #endif  // !defined(_STREAM_H)